{*******************************************************************************
 Project       : "Space Shooter ..." ?
 Date          : 2007-07-11
 Version       : 1.3
 Compiler Ver. : Delphi 2007
 Author        : Denny Koberling
 E-Mail        : JasonVoorhees2k@arcor.de
 Copyright     : Copyright (c) 2007 Denny Koberling
*******************************************************************************}

unit UEngine;

interface

uses
  Messages, Windows, SysUtils, Graphics, Math, Contnrs,
  USprite, UBackground, UStars, UPlayer, UAnimBmp, UType, USpriteList;

type
  TEngine = class(TObject)
  private
    fFrame     : TBitmap;
    fBackground: TBackground;
    fStars     : TStars;
    fHUD1      : TSprite;
    fHUD2      : TSprite;
    fPlayer1   : TPlayer1;
    fSpriteList: TSpriteList;
    function Collision(aSprite1, aSprite2: TSprite): Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure ClearFrame;
    procedure Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse);
    procedure Draw;
    procedure SetSize;
    property Frame: TBitmap read fFrame;
    property Player1: TPlayer1 read fPlayer1;
    property SpriteList: TSpriteList read fSpriteList;
  end;

var
  Engine: TEngine;

implementation

{ TEngine }

constructor TEngine.Create;
begin
  inherited Create;
  fFrame:=TBitmap.Create;
  fFrame.Width:=SCREEN_WIDTH;
  fFrame.Height:=SCREEN_HEIGHT;
  fFrame.PixelFormat:=pf16bit;

  fSpriteList:=TSpriteList.Create;

  fBackground:=TBackground.Create;
  fStars:=TStars.Create;
  fHUD1:=TSprite.Create(nil, 'HUD1', 0, SCREEN_WIDTH-GetBitmap('HUD1').Width, 0);
  fHUD2:=TSprite.Create(nil, 'HUD2', 0, 0, SCREEN_HEIGHT-GetBitmap('HUD2').Height);
  fPlayer1:=TPlayer1.Create(nil, SCREEN_WIDTH div 2, SCREEN_HEIGHT div 2, 0.4);

  fSpriteList.AddSprite(fPlayer1);
  fSpriteList.AddSprite(fHUD1);
  fSpriteList.AddSprite(fHUD2);
  fSpriteList.AddSprite(fBackground);
  fSpriteList.AddSprite(fStars);
end;

destructor TEngine.Destroy;
begin
  FreeAndNil(fSpriteList);
  FreeAndNil(fFrame);
  inherited Destroy;
end;

procedure TEngine.ClearFrame;
var
  i: Integer;
  p: PWord;
begin
  SetSize;
  p:=fFrame.ScanLine[SCREEN_HEIGHT-1];
  for i:=1 to SCREEN_WIDTH*SCREEN_HEIGHT do begin
    p^:=cBlack;
    Inc(p);
  end;
end;

procedure TEngine.Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse);
var
  i, j: Integer;
begin
  // bewegen
  if fSpriteList.Count>0 then
    for i:=0 to fSpriteList.Count-1 do
      fSpriteList[i].Move(aMoveCount, aKeys, aMouse);
  // lschen
  fSpriteList.DeleteDeadSprites;
  // Collision
  if fSpriteList.Count>1 then
    for i:=0 to fSpriteList.Count-2 do
      for j:=i+1 to fSpriteList.Count-1 do
        if fSpriteList[i].AllowHit(fSpriteList[j]) and
           fSpriteList[j].AllowHit(fSpriteList[i]) and
           Collision(fSpriteList[i], fSpriteList[j]) then begin
          fSpriteList[i].Hit(fSpriteList[j]);
          fSpriteList[j].Hit(fSpriteList[i]);
        end;
  // lschen
  fSpriteList.DeleteDeadSprites;
end;

procedure TEngine.SetSize;
begin
  fFrame.Width:=SCREEN_WIDTH;
  fFrame.Height:=SCREEN_HEIGHT;
  fHUD1.X:=SCREEN_WIDTH-GetBitmap('HUD1').Width;
  fHUD2.Y:=SCREEN_HEIGHT-GetBitmap('HUD2').Height;
end;

procedure TEngine.Draw;
var
  i    : Integer;
  Layer: Integer;
begin
  if fSpriteList.Count>0 then
    for Layer:=9 downto 0 do
      for i:=0 to fSpriteList.Count-1 do
        if fSpriteList[i].Layer=Layer then
          fSpriteList[i].Draw(fFrame);
end;

function TEngine.Collision(aSprite1, aSprite2: TSprite): Boolean;
var
  R1, R2, Dest: TRect;
  X, Y: Integer;
begin
  Result:=False;
  R1:=aSprite1.ClipRect;
  R2:=aSprite2.ClipRect;
  if RectInRect(R1, R2) or RectInRect(R2, R1) then begin
    with Dest do begin
      Left:=Max(R1.Left, R2.Left);
      Top:=Max(R1.Top, R2.Top);
      Right:=Min(R1.Right, R2.Right);
      Bottom:=Min(R1.Bottom, R2.Bottom);
    end;
    for Y:=Dest.Top+1 to Dest.Bottom-1 do begin
      for X:=Dest.Left+1 to Dest.Right-1 do
        if (aSprite1.Pixel[X-R1.Left, Y-R1.Top]^.Mask=255) and (aSprite2.Pixel[X-R2.Left, Y-R2.Top]^.Mask=255) then begin
          Result:=True;
          Break;
        end;
      if Result then
        Break;
    end;
  end;
end;

end.
