unit UMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, UCounter, UType, Vcl.StdCtrls, UTimer;

type
  TForm1 = class(TForm)
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormActivate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    Counter: TCounter;
    Keys: TKeys;
    Mouse: TMouse;
    enemyTimer: TTimer;
    infoTimer: TTimer;
    procedure OnRender(aMoveCount: Single);
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  UEngine, UEnemy;

procedure TForm1.OnRender(aMoveCount: Single);
begin
  if enemyTimer.Fire(aMoveCount) then begin
    case Random(4) of
      0: Engine.SpriteList.AddSprite(TEnemy1.Create(nil, -50, -50, 0.1));
      1: Engine.SpriteList.AddSprite(TEnemy2.Create(nil, Random(SCREEN_WIDTH-50), -100, 0.2));
      2: Engine.SpriteList.AddSprite(TEnemy3.Create(nil, SCREEN_WIDTH, Random(200), 0.1));
      3: Engine.SpriteList.AddSprite(TEnemy4.Create(nil, -100, 0, 0.2));
    end;
  end;

  Engine.ClearFrame;
  Engine.Move(aMoveCount, Keys, Mouse);
  Engine.Draw;

  if infoTimer.Fire(aMoveCount) then
    Caption := 'Space (by Denny Koberling), FPS: ' + IntToStr(Round(1000/aMoveCount)) + ', Sprites: ' +
               IntToStr(Engine.SpriteList.Count) + ', EnemyHitCount: '+IntToStr(Engine.Player1.EnemyHitCount) +
               ', PlayerHitCount: '+IntToStr(Engine.Player1.PlayerHitCount);
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  OnActivate := nil;
  while not Keys[VK_ESCAPE] do begin
    if Counter.Elapsed>1000/MAXFPS then begin
      OnRender(Counter.Stop);
      Image1.Picture.Bitmap:=Engine.Frame;
    end else
      Application.ProcessMessages;
  end;
  Close;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Keys[VK_ESCAPE] := True;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  SetThreadPriority(GetCurrentThread, THREAD_PRIORITY_ABOVE_NORMAL);
  Randomize;
  Path:=ExtractFilePath(ParamStr(0));
  Cursor:=-1;

  SCREEN_WIDTH := ClientWidth;
  SCREEN_HEIGHT := ClientHeight;

  Engine:=TEngine.Create;
  Counter:=TCounter.Create;
  enemyTimer:=TTimer.Create(2000);
  infoTimer:=TTimer.Create(200);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(infoTimer);
  FreeAndNil(enemyTimer);
  FreeAndNil(Engine);
  FreeAndNil(Counter);
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  Keys[Key]:=True;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  Keys[Key]:=False;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  SCREEN_WIDTH := ClientWidth;
  SCREEN_HEIGHT := ClientHeight;
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then
    Mouse.RButton := True;
  if Button = mbLeft then
    Mouse.LButton := True;
end;

procedure TForm1.Image1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  Mouse.Pos.X:=X;
  Mouse.Pos.Y:=Y;
end;

procedure TForm1.Image1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then
    Mouse.RButton := False;
  if Button = mbLeft then
    Mouse.LButton := False;
end;

end.
