{*******************************************************************************
 Project       : "Space Shooter ..." ?
 Date          : 2007-07-11
 Version       : 1.3
 Compiler Ver. : Delphi 2007
 Author        : Denny Koberling
 E-Mail        : JasonVoorhees2k@arcor.de
 Copyright     : Copyright (c) 2007 Denny Koberling
*******************************************************************************}

unit UPlayer;

interface

uses Windows, Math, USprite, UType;

type
  TPlayer = class(TSprite);

  TPlayer1 = class(TPlayer)
  private
    fEnemyHitCount: Integer;
    fPlayerHitCount: Integer;
  public
    constructor Create(aParent: TSprite; aX, aY, aSpeed: Single);
    procedure Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse); override;
    procedure Hit(aSprite: TSprite); override;
    function AllowHit(aSprite: TSprite): Boolean; override;
    property EnemyHitCount: Integer read fEnemyHitCount write fEnemyHitCount;
    property PlayerHitCount: Integer read fPlayerHitCount write fPlayerHitCount;
  end;

implementation

uses UTimer, UEngine, UEnemy, UWeapon, UEffect;

{ TPlayer }

constructor TPlayer1.Create(aParent: TSprite; aX, aY, aSpeed: Single);
begin
  inherited Create(aParent, 'Player', 4, aX, aY, aSpeed);
  FireTimer:=TTimer.Create(200);
end;

procedure TPlayer1.Move(aMoveCount: Single; aKeys: TKeys; aMouse: TMouse);
begin
  inherited;

  X:=aMouse.Pos.X - ClipRect.Width div 2;
  Y:=aMouse.Pos.Y - ClipRect.Height div 2;

  {if aKeys[VK_LEFT] then
    X:=X-0.3*aMoveCount;
  if aKeys[VK_RIGHT] then
    X:=X+0.3*aMoveCount;
  if aKeys[VK_DOWN] then
    Y:=Y+0.3*aMoveCount;
  if aKeys[VK_UP] then
    Y:=Y-0.3*aMoveCount;}

  X:=EnsureRange(X,0,SCREEN_WIDTH-1-AnimBmp.Width);
  Y:=EnsureRange(Y,0,SCREEN_HEIGHT-1-AnimBmp.Height);

  if FireTimer.Fire(aMoveCount) and (aKeys[VK_SPACE] or aMouse.LButton) then begin
    Engine.SpriteList.AddSprite(TPlayerWeapon1.Create(Self, X+7, Y+23, 0.4));
    Engine.SpriteList.AddSprite(TPlayerWeapon1.Create(Self, X+52, Y+23, 0.4));
    AnimNr:=1;
  end;
end;

procedure TPlayer1.Hit(aSprite: TSprite);
begin
  X:=EnsureRange(X-5+Random(11),0,SCREEN_WIDTH-1-AnimBmp.Width);
  Y:=EnsureRange(Y-5+Random(11),0,SCREEN_HEIGHT-1-AnimBmp.Height);
  if aSprite is TEnemy then begin
    Inc(fEnemyHitCount);
    Inc(fPlayerHitCount);
  end;
  if aSprite is TEnemyWeapon then
    Inc(fPlayerHitCount);
end;

function TPlayer1.AllowHit(aSprite: TSprite): Boolean;
begin
  Result:=(aSprite is TEnemy) or (aSprite is TEnemyWeapon);
end;

end.
